# -*- coding: utf-8 -*-


class ASF_Error(Exception):

    def __init__(self, message):
        super(ASF_Error, self).__init__()
        self.message = message

    def __repr__(self):
        classname = self.__class__.__name__
        return '{0}({1})'.format(classname, repr(self.message))

    def __str__(self):
        classname = self.__class__.__name__
        return '{0}: {1}'.format(classname, self.message)


class ASF_ResponseError(ASF_Error):

    def __init__(self, message, code=None):
        super(ASF_ResponseError, self).__init__(message)
        self.code = code

    def __repr__(self):
        classname = self.__class__.__name__
        if self.code:
            return '{0}({1}, {2})'.format(classname, repr(self.message), repr(self.code))
        return '{0}({1})'.format(classname, repr(self.message))

    def __str__(self):
        classname = self.__class__.__name__
        if self.code:
            return '{0}: {1} - {2}'.format(classname, self.code, self.message)
        return '{0}: {1}'.format(classname, self.message)


class ASF_ConnectionError(ASF_Error):

    def __init__(self, url):
        super(ASF_ConnectionError, self).__init__('')
        self.url = url
        self.message = 'Cannot establish connection to: {0}'.format(url)

    def __repr__(self):
        classname = self.__class__.__name__
        return '{0}({1})'.format(classname, repr(self.url))

class ASF_WebsocketNotStarted(ASF_Error):

    pass



# -*- coding: utf-8 -*-
import asyncio
import posixpath
import urllib.parse as urlparse
import requests
import websockets

from . import error
from . import response

__version__ = '1.1.5'
ALLBOT = 'ASF'


class IPC(object):

    def __init__(self, ipc='http://127.0.0.1:1242/', password='', timeout=5):
        self.ipc = ipc
        self.password = password
        self.timeout = timeout
        self.wslog = None

    @classmethod
    def _botjoin(cls, bots):
        botnames = bots
        if isinstance(botnames, (list, tuple, set)):
            botnames = ','.join(botnames)
        return botnames

    def _add_auth(self, headers=None):
        if headers is None:
            auth_headers = dict()
        else:
            auth_headers = headers
        if self.password:
            auth_headers['Authentication'] = self.password
        return auth_headers

    def _build_endpoint(self, endpoint, keyword=None, ws=False):
        ipc_split = list(urlparse.urlsplit(self.ipc))
        ipc_split[2] = posixpath.join(ipc_split[2], 'Api', endpoint)
        if keyword:
            keyword = urlparse.quote_plus(keyword)
            ipc_split[2] = posixpath.join(ipc_split[2], keyword)
        if ws and ipc_split[0] == 'http':
            ipc_split[0] = 'ws'
        elif ws and ipc_split[0] == 'https':
            ipc_split[0] = 'wss'
        return urlparse.urlunsplit(ipc_split)

    def get(self, endpoint, keyword=None, headers=None):
        url = self._build_endpoint(endpoint, keyword)
        headers = self._add_auth(headers)
        try:
            resp = requests.get(url, headers=headers, timeout=self.timeout)
        except requests.exceptions.ConnectionError:
            raise error.ASF_ConnectionError(url)
        resp = response.GenericResponse(resp)
        return resp.result

    def delete(self, endpoint, keyword=None, headers=None):
        url = self._build_endpoint(endpoint, keyword)
        headers = self._add_auth(headers)
        try:
            resp = requests.delete(url, headers=headers, timeout=self.timeout)
        except requests.exceptions.ConnectionError:
            raise error.ASF_ConnectionError(url)
        resp = response.GenericResponse(resp)
        return resp.result

    def post(self, endpoint, keyword=None, headers=None, body=None, is_json=False):
        url = self._build_endpoint(endpoint, keyword)
        headers = self._add_auth(headers)
        try:
            if is_json:
                headers['Content-Type'] = 'application/json'
                resp = requests.post(url, json=body, headers=headers, timeout=self.timeout)
            else:
                resp = requests.post(url, data=body, headers=headers, timeout=self.timeout)
        except requests.exceptions.ConnectionError:
            raise error.ASF_ConnectionError(url)
        resp = response.GenericResponse(resp)
        return resp.result

    def post_json(self, endpoint, keyword=None, headers=None, body=None):
        return self.post(endpoint, keyword, headers, body, is_json=True)

    def get_asf(self):
        return self.get('ASF')

    def post_asf(self, global_config, **kwargs):
        payload = {
            'GlobalConfig': global_config
        }
        for key, value in kwargs.items():
            payload[key] = value
        return self.post_json('ASF', body=payload)

    def get_bot(self, botnames):
        botnames = self._botjoin(botnames)
        return self.get('Bot', botnames)

    def delete_bot(self, botnames):
        botnames = self._botjoin(botnames)
        return self.delete('Bot', botnames)

    def post_bot(self, botname, bot_config, **kwargs):
        payload = {
            'BotConfig': bot_config,
            'KeepSensitiveDetails': True
        }
        for key, value in kwargs.items():
            payload[key] = value
        return self.post_json('Bot', botname, body=payload)

    def post_command(self, cmd):
        return self.post('Command', cmd)

    def command(self, cmd):
        return self.post_command(cmd)

    def get_structure(self, structure_name):
        return self.get('Structure', structure_name)

    def get_type(self, type_name):
        return self.get('Type', type_name)

    def post_games_to_redeem_in_background(self, botname, games, **kwargs):
        payload = {
            'GamesToRedeemInBackground': games
        }
        for key, value in kwargs.items():
            payload[key] = value
        return self.post_json('GamesToRedeemInBackground', botname, body=payload)

    def redeem(self, botname, games, **kwargs):
        return self.post_games_to_redeem_in_background(botname, games, **kwargs)

    async def start_log(self):
        await self.stop_log()
        ws_url = self._build_endpoint('Log', ws=True)
        headers = self._add_auth()
        self.wslog = websockets.connect(ws_url, extra_headers=headers)

    async def stop_log(self):
        if self.wslog is None:
            return
        await self.wslog.ws_client.close()
        self.wslog = None

    async def get_log(self):
        if self.wslog is None:
            raise error.ASF_WebsocketNotStarted('Websocket client isn\'t started, try `start_log()` first?')
        async with self.wslog as websocket:
            while True:
                try:
                    resp = await websocket.recv()
                    resp = response.WebsocketResponse(resp)
                    yield resp.result
                except websockets.exceptions.ConnectionClosed:
                    yield None



# -*- coding: utf-8 -*-
import json
from http.client import responses

from . import error


class GenericResponse(object):

    def __init__(self, response):
        code = response.status_code
        message = ''
        try:
            json_content = response.json()
            self.message = json_content.get('Message')
            self.result = json_content.get('Result')
            self.success = json_content.get('Success')
            if not self.success:
                message = self.message
        except ValueError:
            self.message = ''
            self.result = response.text
            self.success = False
            message = responses[code]
        if not self.success:
            raise error.ASF_ResponseError(message, code)

class WebsocketResponse(object):

    def __init__(self, response):
        message = ''
        try:
            json_content = json.loads(response)
            self.message = json_content.get('Message')
            self.result = json_content.get('Result')
            self.success = json_content.get('Success')
            if not self.success:
                message = self.message
        except ValueError:
            self.message = ''
            self.result = response.text
            self.success = False
            message = response.text
        if not self.success:
            raise error.ASF_ResponseError(message)



# -*- coding: utf-8 -*-
import codecs
import os
import re
from setuptools import setup


def read(*parts):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, *parts), 'r') as fp:
        return fp.read()

def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")

setup(
    name='ASF_IPC',
    version=find_version("ASF_IPC", "__init__.py"),
    packages=['ASF_IPC'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    python_requires='>=3.6, <4',
    install_requires=[
        'requests',
        'websockets'
    ],
    author='deluxghost',
    author_email='deluxghost@gmail.com',
    description='A simple Python 3.6+ library of ArchiSteamFarm IPC API',
    long_description='Check GitHub page for usages.',
    license='GPL v3',
    url='https://github.com/deluxghost/ASF_IPC/'
)
